package org.eclipse.team.examples.pessimistic;

/*
 * CMUnit.java 2000/07/01
 *
 * Copyright 1999-2000 by Rational Software Corporation,
 * 18880 Homestead Rd., Cupertino, CA 95014, U.S.A
 * All rights reserved.
 */

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.team.core.RepositoryProvider;


/**
 * A CMUnit class is an interface class to storage units. The purpose of 
 * the class is to store run-time source control information about the 
 * storage units. It also provides a small set of get and set operations 
 * which allow access the the underlying storage unit.
 * 
 * @since   1.0
 * @author	Jean-Michel Lemieux
 */
public class CMUnit
{
	// The actual resource corresponding to this unit
	private IResource resource = null;

	// CM settings for model in which this unit lives	 
	private UnitStatusCache.CMModel	model = null;
/**
 * Don't allow creating default instances of this class.
 */
public CMUnit( IResource newResource )
{
	resource = newResource;
}
///**
// * Insert the method's description here.
// * Creation date: (7/6/2001 12:46:51 PM)
// */
//public boolean doesResourceExist() {
//	return getResource().exists();
//}
///**
// * Allows a unit to perform it's own refresh status. This is provided
// * to support on demand status refreshes. 
// */
//private void doRefresh(boolean force)
//{
//	if( force )
//	{
//		// If model is under source control, and we don't
//		// know the cm status, we should ask for it.
//		if( model.isConnected() && isStatusUnknown() )
//		{
////		
//			Vector units = new Vector();
//			
//			units.addElement( this );
//			
////			ops.queryStatus( units );		
//		}
//	}
//}
public UnitStatusCache.CMModel getCMModel()
{
	return model;
}
/**
 * Insert the method's description here.
 * Creation date: (2/13/2001 10:18:55 AM)
 * @return int
 */
public int getCMStatus() {
	
	return RSCMService.getInstance().getCMStatus( this );
}
/**
 * Insert the method's description here.
 * Creation date: (2/13/2001 10:20:34 AM)
 */
public String getCMStatusAsString() {
	
	int status = getCMStatus();
	
	if ( status == IConstants.RS_Invalid )
	{
		return new String( "Invalid" );
	}
	if ( status == IConstants.RS_Unknown )
	{
		return new String( "Unknown" );
	}	
	if ( status == IConstants.RS_Controlled )
	{
		return new String( "Controlled" );
	}	
	if ( status == IConstants.RS_NotControlled )
	{
		return new String( "Not Controlled" );
	}	
	if ( status == IConstants.RS_CheckedOut )
	{
		return new String( "Checked Out" );
	}	

	return null;	
}
/**
 * Gets the fully qualified name of the element this unit refers to.
 * 
 * @return	the fully qualified name (e.g. Model::Package1::Class1).
 */
public String getFullyQualifiedName()
{
	return getResource().getLocation().toOSString();
}
/** Gets the fully qualified name of the element this unit refers to.
 * 
 * @return	the fully qualified name (e.g. Model::Package1::Class1).
 */
//public String getFullyQualifiedPreviousName()
//{
//	String oldName = (String) getProperty(RSCMService.CM_PREVIOUS_NAME);
//
//	if ( oldName == null )
//	{
//		return getFullyQualifiedName();
//	}
//
//	return oldName;
//}
/**
 * Insert the method's description here.
 * Creation date: (1/31/2001 11:04:04 AM)
 * @return com.rational.startuppkg.solution.RSProject
 */
public IProject getProject() {
	if ( getResource() instanceof IProject )
	{
		return ((IProject)getResource());
	}
	else
	{
		return getResource().getProject();
	}
}
/**
 * Insert the method's description here.
 * Creation date: (6/7/2001 4:57:39 PM)
 */
//public Object getProperty( QualifiedName name ) {
//
//	Object type = null;
//	
//	try {
//		if ( name.equals( RSCMService.ON_DEMAND_ADD_STATUS ) )
//		{
//			QualifiedName newName = new QualifiedName( name.getQualifier(), getFullyQualifiedName() );
//			type = getResource().getPersistentProperty( newName );
//		}
//		else
//		{
//			type = getResource().getSessionProperty( name );
//		}
//	}
//	catch ( CoreException e )
//	{
//		ClearCasePlugin.logError( e.getMessage(), e );
//	}	
//
//	return type;
//}
/**
 * Insert the method's description here.
 * Creation date: (3/7/2001 3:59:42 PM)
 * @return com.rational.startuppkg.solution.RSResource
 */
public IResource getResource() {
	return resource;
}
/**
 * Insert the method's description here.
 * Creation date: (6/11/2001 1:04:42 PM)
 */
//public CMResourceDataAdapter getResourceData() {
//
//	CMResourceDataAdapter details = null;
//	
//	try {
//		details = (CMResourceDataAdapter) getResource().getSessionProperty( RSCMService.RESOURCE_DETAILS );
//	}
//	catch ( CoreException e )
//	{
//		ClearCasePlugin.logError( e.getMessage(), e );
//	}
//
//	return details;
//}
/**
 * Insert the method's description here.
 * Creation date: (6/11/2001 1:04:42 PM)
 */
public boolean isFile() {
	
	return (getResource().getType() == IResource.FILE);
}
/**
 * Insert the method's description here.
 * Creation date: (6/7/2001 6:11:05 PM)
 */
public boolean hasContentChanged() {
	
	boolean returnCode = false;
	
//	if ( getResource().getType() == IResource.FILE )
//	{
//		long time = getResource().getModificationStamp();
//	
//		if ( time != IResource.NULL_STAMP )
//		{
//			Long oldModTime = (Long) getProperty(RSCMService.CM_MOD_TIME);
//		
//			if ( oldModTime != null && time != oldModTime.longValue() )
//			{
//				returnCode = true;
//			}
//		}
//	}
	
	return returnCode;
}
/**
 * Returns the check in status of the file unit.
 * 
 * @return	true if the file unit is checked in, false otherwise.
 */
public boolean isCheckedIn()
{
	PessimisticProvider provider= getProvider(getProject());
	if (provider == null)
		return false;
	return !provider.isCheckedout(resource);
}
/**
 * Returns the check out status of the storage unit.
 * 
 * @return	true if the storage unit is checked out, false otherwise.
 */
public boolean isCheckedOut(boolean force)
{
	PessimisticProvider provider= getProvider(getProject());
	if (provider == null)
		return false;
	return provider.isCheckedout(resource);

}
/**
 * Returns the controlled status of the storage unit.
 * 
 * @return	true if the storage unit is controlled, false otherwise.
 */
public boolean isControlled(boolean force)
{
	return getProvider(getProject()) != null;
}
///**
// * Returns the controlled status of the file unit.
// * 
// * @return	true if the file unit status is unknown, false otherwise
// */
//public boolean isInvalidStatus()
//{
//	return ( getCMStatus() == IConstants.RS_Invalid );		
//}
//public boolean isModelConnected()
//{
//	return model.isConnected();
//}
//public boolean isNotControlled(boolean force)
//{
//	doRefresh(force);
//
//	return ( getCMStatus() == IConstants.RS_NotControlled );		
//}
//public boolean isResourceInBuildOutputFolder()
//{
//	IProject project = resource.getProject();
//	try {
//		// check if the project is a Java Project
//		if (project.hasNature(JavaCore.NATURE_ID)) {
//			IPath resourcePath = resource.getFullPath();
//			IJavaProject javaProject = JavaCore.create(project);
//			IPath projectPath = project.getFullPath();
//			IPath outputLocation = javaProject.getOutputLocation();
//			if (outputLocation.equals(projectPath)) {
//				// if build output path is the project root, return false
//				return false;
//			} else { 
//				// return whether the resource is in the build output path
//				return outputLocation.isPrefixOf(resourcePath);
//			}
//		}
//	} catch (CoreException e) {
//		ClearCasePlugin.logError( e.getMessage(), e );
//	}
//
//	return false;
//}
///**
// * Returns the controlled status of the file unit.
// * 
// * @return	true if the file unit status is unknown, false otherwise
// */
//public boolean isStatusUnknown()
//{
//	return ( getCMStatus() == IConstants.RS_Unknown );		
//}
/**
 * Insert the method's description here.
 * Creation date: (6/7/2001 7:11:36 PM)
 */
public boolean isWritable() {
	return (!resource.isReadOnly());
}
///**
// * Insert the method's description here.
// * Creation date: (8/8/2001 10:16:05 PM)
// */
//public void refreshFromLocal() {
//	try {
//		getResource().refreshLocal(IResource.DEPTH_ZERO, null);
//	}
//	catch (CoreException e )
//	{
//		PessimisticFilesystemProviderPlugin.logError( e.getMessage(), e );
//	}
//}
/**
 * Insert the method's description here.
 * Creation date: (2/5/2001 2:21:49 PM)
 * @param newModel com.rational.startuppkg.cm.UnitStatusCache.CMModel
 */
public void setModel(UnitStatusCache.CMModel newModel) {
	model = newModel;
}
/**
 * Insert the method's description here.
 * Creation date: (6/7/2001 4:57:39 PM)
 */
//public void setProperty( QualifiedName name, Object object ) {
//
//	try {
//		if ( name.equals( RSCMService.ON_DEMAND_ADD_STATUS ) )
//		{
//			QualifiedName newName = new QualifiedName( name.getQualifier(), getFullyQualifiedName() );
//			String propertyValue = getResource().getPersistentProperty( newName );
//			
//			if ( propertyValue != null )
//			{
//				if (!propertyValue.equals( RSCMService.FALSE_AS_STRING ))
//				{
//					getResource().setPersistentProperty( newName, (String) object );
//				}
//			}
//			else
//			{
//				getResource().setPersistentProperty( newName, (String) object );
//			}
//		}
//		else
//		{
//			getResource().setSessionProperty( name, object );
//		}
//	}
//	catch ( CoreException e )
//	{
//		ClearCasePlugin.logError( e.getMessage(), e );
//	}
//}
///**
// * Insert the method's description here.
// * Creation date: (6/7/2001 5:36:10 PM)
// * @param newResourceData com.rational.rcsi.IRSResourceData
// */
//public void setResourceData( CMResourceDataAdapter newResourceData ) {
//
//	try {
//		getResource().setSessionProperty( RSCMService.RESOURCE_DETAILS, newResourceData );
//	}
//	catch ( CoreException e )
//	{
//		ClearCasePlugin.logError( e.getMessage(), e );
//	}
//}
///**
// * Sets the source control status of the file.
// * 
// * @param	the new status for this file.
// */
//public void setStatus( int status )
//{
////	int oldStatus = getCMStatus();
////	if ( oldStatus != status )
////	{
////		setProperty( RSCMService.CM_STATUS, new Integer( status ) );
////
////		if ( status != ICMConstants.RS_Invalid )
////		{
////			setProperty( RSCMService.CM_STATUS_CHANGED, RSCMService.TRUE_AS_STRING );
////		}
////	}
//}
///**
// * Insert the method's description here.
// * Creation date: (2/20/2001 8:44:39 AM)
// * @return boolean
// */
//public void updateModifiedTime() {
//	
//	long time = getResource().getModificationStamp();
//	
//	if ( time != IResource.NULL_STAMP )
//	{
//		setProperty( RSCMService.CM_MOD_TIME, new Long( time ) );
//	}
//}

public PessimisticProvider getProvider(IProject project) {
	return (PessimisticProvider)RepositoryProvider.getProvider(project, PessimisticProvider.ID);
}
}
