
package org.eclipse.team.examples.pessimistic;
import java.util.Hashtable;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFileModificationValidator;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;




public class PessimisticFileModificationValidator
	implements IFileModificationValidator {

	/*
	 * @see IFileModificationValidator#validateEdit(IFile[], Object)
	 */

 	public IStatus validateEdit(IFile[] files, Object context) {
//        	ClearCasePlugin.getPluginDescriptor().getUniqueIdentifier();
    
    	// Empty file list sent, return OK
    	if (files.length == 0) { 
	        return new Status( IStatus.OK, getUid(), IStatus.OK, "OK", null);
	    } 
	
	    //**************************************************
	    // Local declarations and initialization
	    //**************************************************
	
		int checkoutAnswer = 1;    // 0(prompt), 1(yes), 2(no)	
		int someFailAnswer = 1;    // 0(None), 1(ThoseYouCan), 2(AskUser)
	    CMUnit unit = null;
//		CMResourceDataAdapter unit= null;
	    boolean liveContext = (context != null); //Allows debugger override
	
	    Vector checkOutList = new Vector();
	  	int readOnlyCount = 0;     // Count of checked-out, readonly files
		int reloadCount = 0;       // Count of checked-out, need to reload files
	  	int checkoutFailCount = 0; // Count of check-outs that failed
	
	    Hashtable checkoutResourceHash = new Hashtable();
	    
	    // get a handle to the cm service
	    RSCMService cmService = getService();
	
	    CMResourceDataAdapter[] arrayOfFiles = 
	        new CMResourceDataAdapter[ files.length ];
    
	    //****************************************************
	    // Convert original File array to local Resource array
	    //****************************************************
	
	    for ( int i=0; i < arrayOfFiles.length; i++ ) {
	        arrayOfFiles[i] = new CMResourceDataAdapter( files[i]);
	    }
	
	    Vector listOfUnits = 
	    	cmService.convertToCMUnits( files, true );
	
        //**************************************************
	    // StatusCheck - Process resources status loop 
	    //****************************************************
	
	    for ( int i = 0 ; i < arrayOfFiles.length; i++ ) {
	   //  	int localPref = arrayOfFiles[i].getCmPreferences();
	        unit = (CMUnit) listOfUnits.elementAt(i);
			//unit= arrayOfFiles[i];
	
	        //**************************************************
	        // View Private
	        //**************************************************
	
	       if ( !unit.isControlled(false) ) {
		    	//Issue: Add test for writeable before success
	            arrayOfFiles[i].setValidateEditStatus( IConstants.RS_OkToEdit );
	        }
	
	        //**************************************************
	        // File is not already checked out 
	        // Accumulate list of files we want to try to checkout
	        //**************************************************
	
	        else if ( !unit.isCheckedOut(false ) ) {
	            checkOutList.addElement( unit );
	            checkoutResourceHash.put( unit.getResource(), arrayOfFiles[i]);
	        }
	
	        //**************************************************
	        // Catch case of read-only Checked out files
	        //**************************************************

			else if ( !unit.isWritable() ) {
//            	ClearCasePlugin.logError( 
//            	    "CCFileModificationValidator::validateEdit: " + 
//            	    unit.getFullyQualifiedName() +
//            	    " is checked out but has read only properties", null );

            	readOnlyCount++;
            	arrayOfFiles[i].setValidateEditStatus( 
                	IConstants.RS_NotOkToEdit );
//            	arrayOfFiles[i].setValidateEditStatusDetails( 
//                	ICMConstants.RS_EditNotPossible);
        	}

	        //**************************************************
	        // Otherwise checked out and writeable
	        //**************************************************
	
	        else {
	            arrayOfFiles[i].setValidateEditStatus( 
	               IConstants.RS_OkToEdit );
//	            arrayOfFiles[i].setValidateEditStatusDetails( 
//	               ICMConstants.RS_CheckedOutForEdit);
	        }
	    }
	
	    //**************************************************
	    // At least some files need to be checked out
	    //**************************************************
	
	    if (checkOutList.size() != 0) {
	
     		Integer noPromptActionInt = (Integer) cmService.getPreference( 
				IConstants.RM_WhenCheckedInFilesAreEditedNoPrompt,
				IConstants.RM_INT);
			int noPromptAction = noPromptActionInt.intValue();


	        //**************************************************
            // If Context is valid then use cmService.onDemand
            // which will prompt for user decisions in CC layer
	        //**************************************************

			boolean fail= isFailValidateEdit();	
	     	if (liveContext) {

	            // Request checkout of everything not already checked out	
	            cmService.onDemand( checkOutList, 
	                      IConstants.RM_WhenCheckedInFilesAreEdited, fail );
	            if (fail) {
					return new Status( IStatus.ERROR, getUid(), IStatus.ERROR, "Fail Validate Edit Preference true", null);
				}	                      
            }
            else if (noPromptAction == 0) { // Automatic checkout preference
	            if (fail) {
					return new Status( IStatus.ERROR, getUid(), IStatus.ERROR, "Fail Validate Edit Preference true", null);
				}	                      

				for ( int i = 0 ; i < checkOutList.size() ; i++ ) {
					unit = (CMUnit) checkOutList.elementAt(i);
//					CMResourceDataAdapter resourceData = 
//						(CMResourceDataAdapter) checkoutResourceHash.get( unit.getResource() );
//					int result = checkOut(unit);
					IFile file= null;
					if (unit.getResource().getType() == IResource.FILE) {
						getService().checkout(new IFile[] {(IFile) unit.getResource()});
					}
//					if (unit.is ) {  //set the cm status
//						unit.setStatus(ICMConstants.RS_CheckedOut);
//					}
				}
            }
	        //**************************************************
	        // Loop to post-process results of checkout attempt
	        //**************************************************
	
	        for ( int i = 0 ; i < checkOutList.size() ; i++ ) {
				unit = (CMUnit) checkOutList.elementAt(i);
				CMResourceDataAdapter resourceData = 
			    	(CMResourceDataAdapter) checkoutResourceHash.get( unit.getResource() );
	        
				if ( unit.isCheckedOut( false ) ) {
			    	if ( !unit.hasContentChanged() ) {
						resourceData.setValidateEditStatus( 
				    		IConstants.RS_OkToEdit );
//	                    resourceData.setValidateEditStatusDetails( 
//	                        ICMConstants.RS_CheckedOutForEdit);
			    	} else {
	                    reloadCount++;
						resourceData.setValidateEditStatus( 
	                        IConstants.RS_PromptForReloadBeforeEdit );
//						resourceData.setValidateEditStatusDetails( 
//	                        ICMConstants.RS_ChangedOnCheckOutForEdit);
			    	}
				}
	
				//Failed to checkout the file
				else { 
	                checkoutFailCount++;
			    	resourceData.setValidateEditStatus( 
	                    IConstants.RS_NotOkToEdit );
//			    	resourceData.setValidateEditStatusDetails( 
//	                    ICMConstants.RS_CheckoutCancelledOrFailedForEdit);
				}
		    }	
	    }
	
	    //**************************************************
	    // Determine return status, using single status if possible    
	    //**************************************************
	
		//Do a refresh
//		UnitStatusCache cache = new UnitStatusCache();
//		cache.addUnits(listOfUnits);		
//		RSCMService.getInstance().updateStatus(cache);
	
	    // All succeeded
	    if (readOnlyCount + reloadCount + checkoutFailCount == 0) { 
	        return new Status( IStatus.OK, getUid(), IStatus.OK, "OK", null);
	    }
	
	    // All failed
	    if (readOnlyCount + checkoutFailCount == files.length) {
	        return new Status( IStatus.ERROR, getUid(), IStatus.ERROR, "NOTOK", null);
	    }
	
	    // Build up multi-status return
	    IStatus children[] = new Status[ files.length ];
	
	    int mask = IConstants.RS_OkToEdit |
	               	IConstants.RS_PromptForReloadBeforeEdit;
	
	    for (int i = 0; i < files.length; i++) { 
	    	int result = arrayOfFiles[i].getValidateEditResult();
	 		if ((result & mask) != 0) {
		    	children[i] = new Status( IStatus.OK, getUid(), IStatus.OK, "OK", null);
	        } else {
	            children[i] = new Status( IStatus.ERROR, getUid(), IStatus.ERROR, "NOTOK", null);
	        }
	    }
	    return new MultiStatus( getUid(), IStatus.OK, children, "MULTISTATUS", null); 
	}

	/**
	 * @see IFileModificationValidator#validateSave(IFile)
	 */
	public IStatus validateSave(IFile file) {
		IStatus status = new Status( IStatus.OK, getUid(), IStatus.OK, 
			                      	"File is writeable", null);
		
//    	java.lang.String uid = 
//        	ClearCasePlugin.getPluginDescriptor().getUniqueIdentifier();
// 
//    	IStatus status = null;
//
//    	// return null if file is null
//    	if ( file == null ) { return null; }

		// get the cm status
		CMUnit unit = new CMUnit( file );
		
		if (unit.isCheckedOut(false)) {
			return status;
		}
		
		Vector units = new Vector();
		units.add(unit);
		
		getService().onDemand(units, IConstants.RM_WhenCheckedInFilesAreSaved, false);
		
		if (unit.isCheckedOut(false)) {
			return status;
		} else {
			return new Status( IStatus.ERROR, getUid(), IStatus.ERROR, 
                                 	"File could not be checked out", null);
		}
		

//		// check if unit is controlled
//		if ( !unit.isControlled(false) ) { // not under source control
//			if ( unit.isWritable() ) {
//				status = new Status( IStatus.OK, uid, IStatus.OK, 
//			                      	"File is writeable", null);
//			} else {
//				status = new Status( IStatus.ERROR, uid, IStatus.ERROR, 
//			                      	"File is read only", null);
//			}
//		} else if ( !unit.isCheckedOut(false) ) { // checked in file
//			// update the mode time
//			unit.updateModifiedTime();
//		
//			int result = ClearCaseConnection.calCheckOut(unit.getFullyQualifiedName());
//
//			if ( result == 0 )
//			{
//				//set the cm status
//				unit.setStatus(ICMConstants.RS_CheckedOut);
//
//				// set the return value
//				status = new Status( IStatus.OK, uid, IStatus.OK, 
//			                      	"File is checked out", null);
//				// log
//				ClearCasePlugin.logTrace( "Checked out " + 
//			    	unit.getFullyQualifiedName() + "...", null);
//			
//				// update the toolbar
//				RSCMService cmService = (RSCMService) ClearCasePlugin.getCMService();
//				Vector list = new Vector();
//				list.addElement(unit);
//				cmService.updateUI( list );
//			} else {
//				status = new Status( IStatus.ERROR, uid, IStatus.ERROR, 
//                                 	"File could not be checked out", null);
//			}
//		} else if ( !unit.isWritable() ) { // hijacked file
//			status = new Status( IStatus.ERROR, uid, IStatus.ERROR, 
//		                     	"File is checked out but read only", null );
//		} else { // already checked out
//			status = new Status( IStatus.OK, uid, IStatus.OK,
//							 	"File is already checked out", null );
//		}
//	
//		//Refresh the file's anotation icon
//		IResource resource = unit.getResource();
//	
//		return status;
	}
	
	private String getUid() {
		return PessimisticFilesystemProviderPlugin.PI_ID;
	}
	
	private RSCMService getService() {
		return RSCMService.getInstance();
	}

	private boolean isFailValidateEdit() {
     	return ((Boolean) getService().getPreference( 
				IConstants.RM_FailValidateEdit,
				IConstants.RM_BOOLEAN)).booleanValue();
	}		
}
