package org.eclipse.team.examples.pessimistic;

import java.net.URL;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.plugin.AbstractUIPlugin;

public class PessimisticFilesystemProviderPlugin extends AbstractUIPlugin {
	private static PessimisticFilesystemProviderPlugin instance;
	
	/**
	 * The id of the FileSystem plug-in
	 */
	public static final String PI_ID = "org.eclipse.team.examples.pessimistic";
	
	/**
	 * The identifier for the File System nature
	 *
	 * @see org.eclipse.core.resources.IProject#hasNature
	 */
	public static final String FSNATURE_ID = PI_ID + ".PessimisticSimpleProvider";
	
	/**
	 * The identifier for the WebDAV nature
	 *
	 * @see org.eclipse.core.resources.IProject#hasNature
	 */
//	public static final String DAVNATURE_ID = PI_ID + ".WebDavProvider";

	/** Constructs a plug-in runtime class for the given plug-in descriptor.
	 */
	public PessimisticFilesystemProviderPlugin(IPluginDescriptor pluginDescriptor) {
		super(pluginDescriptor);
		instance = this;
	}
	/**
	 * Return the File System plugin instance
	 */
	public static PessimisticFilesystemProviderPlugin getInstance() {
		return instance;
	}
	
/** 
 * Sets default preference values. These values will be used
 * until some preferences are actually set using Preference dialog.
 */
protected void initializeDefaultPreferences(IPreferenceStore store) {
	// These settings will show up when Preference dialog
	// opens up for the first time.
//	store.setDefault(ICMInternalConstants.RM_AllowInMemoryEdits, false);
//	store.setDefault(ICMInternalConstants.RM_ListenToFileSystemChanges, false);
//	store.setDefault(ICMInternalConstants.RM_DoNotShowVersionMsg, false);	
//	store.setDefault(ICMInternalConstants.RM_WhenCheckedInFilesAreEdited, ICMInternalConstants.RM_PROMPT);
//	store.setDefault(ICMInternalConstants.RM_WhenCheckedInFilesAreSaved, ICMInternalConstants.RM_PROMPT);
//	store.setDefault(ICMInternalConstants.RM_WhenNewResourcesAreAdded, ICMInternalConstants.RM_PROMPT);
	store.setDefault(IConstants.RM_WhenCheckedInFilesAreEdited, IConstants.RM_AUTOMATIC);
	store.setDefault(IConstants.RM_WhenCheckedInFilesAreSaved, IConstants.RM_PROMPT);
//	store.setDefault(ICMInternalConstants.RM_WhenNewResourcesAreAdded, ICMInternalConstants.RM_PROMPT);
//	store.setDefault(ICMInternalConstants.RM_WhenWorkspaceIsClosed, ICMInternalConstants.RM_PROMPT);
//	store.setDefault(ICMInternalConstants.RM_IgnoreResources, ICMInternalConstants.RM_DEFAULT_PATTERNS);	
	store.setDefault(IConstants.RM_FailValidateEdit, false);
	store.setDefault(IConstants.RM_ChangeFileContents, false);	
}
	
	public void startup() throws CoreException {
	}	
	
	public static void log(IStatus status) {
	}
	public static void logError(String message, Throwable throwable) {
	}

	public static RSCMService getCMService() {
		return RSCMService.getInstance();
	}
	
	
	public static PessimisticFilesystemProviderPlugin getPlugin() {
		return instance;
	}
	
public static Path getInstallLocation() {

	String installLocation = null;

	try {
		URL url = getPlugin().getDescriptor().getInstallURL();
		String path = Platform.resolve(url).getFile();
		
		if ( path.startsWith("/") )
		{
			path = path.substring(1);
		}
		
		path = new Path(path).toOSString();

		if ( path.endsWith(java.io.File.separator) )
		{
			installLocation = path;
		}
		else
		{
			installLocation = path + java.io.File.separator;
		}

		return new Path( installLocation );
	}
	catch ( Exception e )
	{
		return null;
	}
}

public static boolean isEditorFeaturesRequired() {
			
	return false;
}	
}
