package org.eclipse.team.examples.pessimistic;

/*
 * UnitStatusCache.java 2000/07/01
 *
 * Copyright 1999-2000 by Rational Software Corporation,
 * 18880 Homestead Rd., Cupertino, CA 95014, U.S.A
 * All rights reserved.
 */

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.team.examples.pessimistic.*;



/**
 * The UnitStatusCache is a class which allows the addin to maintain
 * CMUnit instances grouped by the model that contains them. This allows
 * for easy lookup of units by model, and allows clients to retreive the
 * cm status information for a model and use this status information for
 * cm operations.
 * 
 * {guid,Model1Info}  
 *           |
 *           +--> {guid,UnitInfo},{guid,UnitInfo}...
 * {guid,Model2Info} 
 *           |
 *			 +--> {guid,UnitInfo},{guif,UnitInfo}...
 *			.
 *			.
 * 
 */
public class UnitStatusCache
{
	// A collection of the models within this cache, each model
	// contains another hash of their storage units.
	private Hashtable models = null;

	// Node class used to track model level CM settings. This node is
	// inserted into the models hashtable.

	public class CMModel
	{
//		private CMModelSettings	settings = null;
		private Hashtable units = null;
		private IProject root = null;
		
		CMModel( IProject rootProject )
		{
			units = new Hashtable();
			root = rootProject;
			RSCMService cmService = (RSCMService) PessimisticFilesystemProviderPlugin.getCMService();
//			settings = (CMModelSettings)cmService.getListOfProjectSettings().get(root);
		}
	
		public void clear()
		{
			units.clear();
		}
		
		public boolean isConnected()
		{
			return true;
//			if (settings == null) 
//				return false;
//			else
//				return settings.getState() == CMState.CONNECTED;	
		}
		
		public IProject getRootUnit()
		{
			return root;
		}
		
//		public CMModelSettings getSettings()
//		{
//			return settings;
//		}
		
//		public void setSettings( CMModelSettings modelSettings )
//		{
//			settings = modelSettings;	
//		}
		
		public void add( CMUnit unit )
		{
			if( !units.containsKey( unit.getResource() ) )
			{
				units.put( unit.getResource(), unit );
			}
		}
		
		public CMUnit remove( CMUnit unit )
		{
			return (CMUnit)units.remove( unit.getResource() );
		}
		
		public CMUnit get( String fullyQualifiedName )
		{
			Enumeration e = units.elements();
			CMUnit unit = null;
			
			while( e.hasMoreElements() )
			{
				unit = (CMUnit) e.nextElement();

				if ( unit.getFullyQualifiedName().equals( fullyQualifiedName ) )
				{
					return unit;
				}
			}
			
			return null;			
		}
		
		public Enumeration elements()
		{
			return units.elements();	
		}
		
		public Vector getAllUnits()
		{
			Vector list = new Vector();
			
			Enumeration e = units.elements();
			
			while( e.hasMoreElements() )
			{
				list.addElement( e.nextElement() );
			}
			
			return list;
		}
		
		public int size()
		{
			return units.size();
		}
		
//		public void disconnect()
//		{
//		}
					
//		public void connect()
//		{
//			Enumeration list = elements();
//			
//			while( list.hasMoreElements() )
//			{
//				((CMUnit)list.nextElement()).setStatus( ICMConstants.RS_Unknown );
//			}	
//		}
		
//		public String toString()
//		{		
//			return new String(  root.getName() + " (" + 
//								settings.getProviderInfo().getName() + ")" );
//		}
	}
/**
 * UnitStatusCache class constructor.
 * 
 * @param VcAddin addin	-- The VcAddin object.
 */
public UnitStatusCache()
{
	models = new Hashtable();
}
/**
 * Use to add a new model to the cache. New models must be added
 * when loaded.
 */
private UnitStatusCache.CMModel addModel( IProject root ) throws IllegalArgumentException
{
	if( models.containsKey( root.getName() ) )
	{
		throw new IllegalArgumentException("Cannot add duplicate models!");	
	}
	
	CMModel model = new CMModel( root );
	models.put( root.getName(), model );
	return model;
}
/**
 * Helper for adding a collection of units to the cache.
 * 
 * @param  units is a collection of CMUnit instances that are to be
 *		   added to the cache.
 */
public void addUnits(Vector units)
{
	CMUnit unit = null;
	CMModel model = null;
	IProject project = null;
	
	for( int i = 0; i < units.size(); i++ )
	{
		unit  = (CMUnit) units.elementAt(i);

		project = unit.getProject();

		model = (CMModel) models.get( project.getName() );

		if ( model == null )
		{
			model = addModel( project );
		}

		unit.setModel( model );
		model.add( unit );
	}
}
public void clear()
{
	for ( Enumeration e = models.elements() ; e.hasMoreElements() ; ) 
	{
		((CMModel)e.nextElement()).clear();
	}

	models.clear();
}
/**
 * Marks all units with in invalid cm source control status.
 */
//public void connect( CMModel model )
//{
//	if( models.containsKey( model.getRootUnit().getName() ) )
//	{
//		CMModel ourModel  = (CMModel) models.get( model.getRootUnit().getName() );
//		ourModel.connect();
//	}
//}
/**
 * Marks all units with in invalid cm source control status.
 */
//public void disconnect(CMModel model)
//{
//	if( models.containsKey( model.getRootUnit().getName() ) )
//	{
//		CMModel ourModel  = (CMModel) models.get( model.getRootUnit().getName() );
//		ourModel.disconnect();
//	}	
//}
/**
 * Return an enumeration of
 */
public Enumeration elements()
{
	return models.elements();
}
/**
 * Returns a collection of all units in the cache.
 */
public Vector getAllUnits()
{
	Vector units = new Vector();
		
	for ( Enumeration e = models.elements() ; e.hasMoreElements() ; ) 
	{
		Enumeration u = ((CMModel)e.nextElement()).elements();
	
		while( u.hasMoreElements() )
		{	
			units.addElement( (CMUnit)u.nextElement() );
		}
	}
		
	return units;
}
/**
 * Insert the method's description here.
 * Creation date: (4/22/2001 2:32:13 PM)
 * @return boolean
 */
//public Vector getModelsBasedOnViewProperty(QualifiedName propName, String propValue) {
//
//	Vector listOfModels = new Vector();
//	UnitStatusCache.CMModel model = null;
//	CMUnit unit = null;
//	String viewPropValue = null;
//
//	if ( !models.isEmpty() )
//	{
//		Enumeration e = models.elements();
//				
//		while ( e.hasMoreElements() )
//		{
//			model = (UnitStatusCache.CMModel) e.nextElement();
//
//			unit = new CMUnit( model.getRootUnit() );
//
//			viewPropValue = (String) unit.getProperty( propName );
//
//			if ( viewPropValue != null && viewPropValue.equals( propValue ) )
//			{
//				listOfModels.addElement(model);
//			}
//		}
//	}
//
//	return listOfModels;
//}
/**
 * Return the status of the cache
 */
public boolean isEmpty()
{
	return models.isEmpty();
}
///**
// * Returns true if at least one model is under source control
// */
//public boolean isOneModelConnected()
//{
//	for ( Enumeration e = models.elements() ; e.hasMoreElements() ; ) 
//	{
//		CMModel model = (CMModel) e.nextElement();
//		
//		if( model.isConnected() )
//			return true;		
//	}
//
//	return false;	 
//}
/**
 * Removes the specified unit from the cache, and returns the removed
 * unit. If the unit was not in the cache, null is returned.
 * 
 * @param  unit to remove from the cache.
 * @return unit that was removed, or null if unit was not in the cache.
 */
public CMUnit remove(CMUnit unit)
{
	CMModel model = (CMModel) models.get( unit.getCMModel().getRootUnit().getName() );
	
	if( model != null )
	{
		CMUnit r_unit = model.remove( unit );
		
		if( model.size() == 0 )
		{
			removeModel( model );
		}
		
		return r_unit;
	}
	else
	{
		return null;
	}
}
/**
 * Removes the specified unit from the cache, and returns the removed
 * unit. If the unit was not in the cache, null is returned.
 * 
 * @param  unit to remove from the cache.
 * @return unit that was removed, or null if unit was not in the cache.
 */
public CMModel removeModel(CMModel model)
{
	return (CMModel) models.remove( model.getRootUnit().getName() );
}
///**
// * Removes a collection of units from the cache.
// * 
// * @param  units is a collection of CMUnit instances that should be
// *		   removed from the cache.
// */
//public void removeUnits(Vector units)
//{
//	CMUnit unit = null;
//	CMModel model = null;
//	
//	for( int i = 0; i < units.size(); i++ )
//	{
//		unit  = (CMUnit) units.elementAt(i);
//
//		model = (CMModel) models.get( unit.getProject().getName() );
//			
//		if ( model != null )
//		{
//			model.remove( unit );
//		
//			if ( model.size() == 0 )
//			{
//				removeModel( model );
//			}
//		}
//	}
//}
/**
 * Returns number of models in cache.
 */
//public int size()
//{
//	return models.size();
//}
/**
 * Return number of units in the cache.
 */
//public int sizeUnits()
//{
//	int size = 0;
//		
//	for ( Enumeration e = models.elements() ; e.hasMoreElements() ; ) 
//	{
//		size = size + ((CMModel)e.nextElement()).size();			
//	}
//	
//	return size;
//}
/**
 * Return number of units in the cache.
 */
public IFile[] getAllFiles()
{
	Vector units = getAllUnits();
	
	if ( units.size() > 0 )
	{
		Vector files = new Vector();
		CMUnit unit = null;

		for ( int i=0; i < units.size(); i++ )
		{
			unit = (CMUnit) units.elementAt(i);
			
			if ( unit.isFile() )
			{
				files.addElement((IFile)unit.getResource());
			}
		}
			
		if ( files.size() > 0 )
		{
			IFile[] array = new IFile[ files.size() ];
			files.copyInto(array);
			return array;
		}
	}
	
	return null;
}
/**
 * Return number of units in the cache.
 */
//public StructuredSelection getAllUnitsAsStructuredSelection()
//{
//	Vector units = getAllUnits();
//	
//	if ( units.size() > 0 )
//	{
//		IResource[] array = new IResource[ units.size() ];
//		for ( int i=0; i < units.size(); i++ )
//		{
//			array[i] = ((CMUnit) units.elementAt(i)).getResource();
//		}
//		
//		return (new StructuredSelection(array));
//	}
//	
//	return new StructuredSelection();
//}
}
