package org.eclipse.team.examples.pessimistic.ui;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.team.examples.pessimistic.IConstants;
import org.eclipse.team.examples.pessimistic.PessimisticFilesystemProviderPlugin;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;


/**
 * Insert the type's description here.
 * Creation date: (8/20/2001 1:22:26 PM)
 * @author: Vishy Ramaswamy
 */
public class PessimisticPreferencesPage extends PreferencePage implements /*Listener, */IWorkbenchPreferencePage {

	protected Button createCheckBox(Composite group, String label) {
		Button button = new Button(group, SWT.CHECK | SWT.LEFT);
		button.setText(label);
		GridData data = new GridData();
		button.setLayoutData(data);
		return button;
	}

	protected Combo createCombo(Composite group) {

		Combo combo = new Combo(group, SWT.BORDER | SWT.READ_ONLY);
		GridData gridData = new GridData();
		gridData.horizontalSpan = 1;
		gridData.horizontalAlignment = GridData.FILL;
		combo.setLayoutData(gridData);

		return combo;
	}

	protected Label createLabel(Composite group, String message, int span) {

		Label label = new Label(group, SWT.NONE);
		label.setText(message);
		GridData gridData = new GridData();
		gridData.horizontalSpan = span;
		gridData.horizontalAlignment = GridData.FILL;
		label.setLayoutData(gridData);

		return label;
	}

	protected IPreferenceStore doGetPreferenceStore() {
		return PessimisticFilesystemProviderPlugin.getPlugin().getPreferenceStore();
	}

	public void setDefaultLayout(Composite group, int columns) {

		GridLayout layout = new GridLayout();
		group.setLayout(layout);

		GridData data =
			new GridData(
				GridData.VERTICAL_ALIGN_FILL
					| GridData.HORIZONTAL_ALIGN_FILL
					| GridData.GRAB_HORIZONTAL);

		layout.numColumns = columns;
		group.setLayoutData(data);
	}


	// attribute for the list of buttons
//	Button allowInMemoryEditsButton;

	// attribute for the list of buttons
	//Button listenToFileSystemChangesButton;

	// list of combo boxes
	Combo filesAreEditedCombo;

	// list of labels
	Label filesAreEditedComboLabel;

//	Combo filesAreEditedNoPromptCombo;

//	Label filesAreEditedNoPromptComboLabel;

	Combo filesAreSavedCombo;

	Label filesAreSavedComboLabel;
	
	Button changeFileContents;
	
	Button failValidateEdit;


	protected Control createContents(Composite parent) {
		// check box group
		Composite composite = new Composite(parent, SWT.NULL);
		setDefaultLayout(composite, 1);

//		Composite options = new Composite(composite, SWT.NULL);
//		setDefaultLayout(options, 1);

//		allowInMemoryEditsButton =
//			createCheckBox(options, "Allow checked in files to be edited");
		//listenToFileSystemChangesButton = createCheckBox(options, "Listen to file system changes");

		
		
		// combo box group
		Composite options = new Composite(composite, SWT.NULL);
		setDefaultLayout(options, 2);

		createLabel(options, "File handling:", 2);
		
		filesAreEditedComboLabel =
			createLabel(options, "When checked in files are edited:", 1);
		filesAreEditedCombo = createCombo(options);
		filesAreEditedCombo.setItems(
			new String[] { "Prompt", "Automatically checkout ", "Do nothing" });

		filesAreSavedComboLabel =
			createLabel(options, "When checked in files are saved:", 1);
		filesAreSavedCombo = createCombo(options);
		filesAreSavedCombo.setItems(
			new String[] { "Prompt", "Automatically checkout ", "Do nothing" });

//		filesAreEditedNoPromptCombo = createCombo(options);
//		filesAreEditedNoPromptCombo.setItems(
//			new String[] { "Automatically checkout ", "Do nothing" });
//		filesAreEditedNoPromptComboLabel =
//			createLabel(options, "When checked in files are edited and can't prompt");

		options= new Composite(composite, SWT.NULL);
		setDefaultLayout(options, 1);
		
		createLabel(options, "Error cases:", 1);
		failValidateEdit=
			createCheckBox(options, "Fail validate edit");
		changeFileContents=
			createCheckBox(options, "Change file content");


		updatePreferencePage();


		return composite;
	}


	public void init(IWorkbench workbench) {
	}

	protected void initializeDefaults() {
		IPreferenceStore store = getPreferenceStore();

//		allowInMemoryEditsButton.setSelection(
//			store.getDefaultBoolean(ICMInternalConstants.RM_AllowInMemoryEdits));
		//listenToFileSystemChangesButton.setSelection(store.getDefaultBoolean(ICMInternalConstants.RM_ListenToFileSystemChanges));
		filesAreEditedCombo.select(
			store.getDefaultInt(IConstants.RM_WhenCheckedInFilesAreEdited));
		filesAreSavedCombo.select(
			store.getDefaultInt(IConstants.RM_WhenCheckedInFilesAreSaved));
//		filesAreEditedNoPromptCombo.select(
//			store.getDefaultInt(
//				ICMInternalConstants.RM_WhenCheckedInFilesAreEditedNoPrompt));

		failValidateEdit.setSelection(
			store.getDefaultBoolean(IConstants.RM_FailValidateEdit));
		changeFileContents.setSelection(
			store.getDefaultBoolean(IConstants.RM_ChangeFileContents));
	}

	protected void performDefaults() {
		super.performDefaults();
		initializeDefaults();
	}

	public boolean performOk() {
		storeValues();
		return true;
	}

	protected void storeValues() {
		IPreferenceStore store = getPreferenceStore();

		//store.setValue(ICMInternalConstants.RM_ListenToFileSystemChanges, listenToFileSystemChangesButton.getSelection());
//		store.setValue(
//			ICMInternalConstants.RM_AllowInMemoryEdits,
//			allowInMemoryEditsButton.getSelection());
		store.setValue(
			IConstants.RM_WhenCheckedInFilesAreEdited,
			filesAreEditedCombo.getSelectionIndex());
		store.setValue(
			IConstants.RM_WhenCheckedInFilesAreSaved,
			filesAreSavedCombo.getSelectionIndex());
//		store.setValue(
//			ICMInternalConstants.RM_WhenCheckedInFilesAreEditedNoPrompt,
//			filesAreEditedNoPromptCombo.getSelectionIndex());

		store.setValue(
			IConstants.RM_FailValidateEdit,
			failValidateEdit.getSelection());
		store.setValue(
			IConstants.RM_ChangeFileContents,
			changeFileContents.getSelection());
			
	}

	protected void updatePreferencePage() {
		IPreferenceStore store = getPreferenceStore();

		//listenToFileSystemChangesButton.setSelection(store.getBoolean(ICMInternalConstants.RM_ListenToFileSystemChanges));		
//		allowInMemoryEditsButton.setSelection(
//			store.getBoolean(ICMInternalConstants.RM_AllowInMemoryEdits));
		filesAreEditedCombo.select(
			store.getInt(IConstants.RM_WhenCheckedInFilesAreEdited));
		filesAreSavedCombo.select(
			store.getInt(IConstants.RM_WhenCheckedInFilesAreSaved));
//		filesAreEditedNoPromptCombo.select(
//			store.getInt(ICMInternalConstants.RM_WhenCheckedInFilesAreEditedNoPrompt));

		failValidateEdit.setSelection(
			store.getBoolean(IConstants.RM_FailValidateEdit));
		changeFileContents.setSelection(
			store.getBoolean(IConstants.RM_ChangeFileContents));

	}

}