package org.eclipse.team.examples.pessimistic.ui;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.examples.pessimistic.PessimisticProvider;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

/**
 * @author jed
 *
 * To change this generated comment edit the template variable "typecomment":
 * Workbench>Preferences>Java>Templates.
 */
public abstract class PessimisticProviderAction
	implements IWorkbenchWindowActionDelegate {

	protected ISelection fSelection;

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
		fSelection= null;
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		fSelection = selection;
		
		boolean enabled= action.isEnabled();
		if (enabled != checkEnablement()) {
			action.setEnabled(!enabled);
		}
	}

	protected boolean checkEnablement() {
		IResource[] resources= getSelectedResources();
		if (resources == null || resources.length == 0) {
			return false;
		} else {
			boolean enabled= true;
			for(int i= 0; enabled && i < resources.length; i++) {
				if (!shouldEnableFor(resources[i])) {
					enabled= false;
				}
			}
			return enabled;
		}
	}
	
	protected abstract boolean shouldEnableFor(IResource resource);
	
	protected IResource[] getSelectedResources() {
		ArrayList resources = null;
		if (!fSelection.isEmpty()) {
			resources = new ArrayList();
			Iterator elements = ((IStructuredSelection) fSelection).iterator();
			while (elements.hasNext()) {
				Object next = elements.next();
				if (next instanceof IResource) {
					resources.add(next);
					continue;
				}
				if (next instanceof IAdaptable) {
					IAdaptable a = (IAdaptable) next;
					Object adapter = a.getAdapter(IResource.class);
					if (adapter instanceof IResource) {
						resources.add(adapter);
						continue;
					}
				}
			}
		}
		if (resources != null && !resources.isEmpty()) {
			IResource[] result = new IResource[resources.size()];
			resources.toArray(result);
			return result;
		}
		return new IResource[0];		
	}
	
	protected boolean isControlled(IResource resource) {
		PessimisticProvider provider= getProvider(resource);
		if (provider == null)
			return false;
		return true;
	}
	
	protected boolean isCheckedOut(IResource resource) {
		PessimisticProvider provider= getProvider(resource);
		if (provider == null)
			return false;
		return provider.isCheckedout(resource);
	}

	protected PessimisticProvider getProvider(IResource resource) {
		RepositoryProvider provider= RepositoryProvider.getProvider(resource.getProject(), PessimisticProvider.ID);
		if (provider instanceof PessimisticProvider) {
			return (PessimisticProvider)provider;
		}
		return null;
	}
		
}
